"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.tsx
var src_exports = {};
__export(src_exports, {
  default: () => src_default2
});
module.exports = __toCommonJS(src_exports);
var import_react2 = require("react");

// src/useCustomScroller.ts
var import_react = require("react");
var OS_SCROLLBAR_WIDTH = (() => {
  const outer = document.createElement("div");
  const inner = document.createElement("div");
  outer.style.overflow = "scroll";
  outer.style.width = "100%";
  inner.style.width = "100%";
  document.body.appendChild(outer);
  outer.appendChild(inner);
  const scrollbarWidth = outer.offsetWidth - inner.offsetWidth;
  outer.removeChild(inner);
  document.body.removeChild(outer);
  return scrollbarWidth;
})();
var SCROLLBAR_WIDTH = OS_SCROLLBAR_WIDTH || 20;
function useCustomScroller(customRef, { disabled } = { disabled: false }) {
  const [scrollRatio, setScrollRatio] = (0, import_react.useState)(1);
  const [isDraggingTrack, setIsDraggingTrack] = (0, import_react.useState)(false);
  const ref = (0, import_react.useRef)(null);
  const scrollerRef = customRef || ref;
  const trackRef = (0, import_react.useRef)(null);
  const trackAnimationRef = (0, import_react.useRef)(0);
  const memoizedProps = (0, import_react.useRef)();
  (0, import_react.useLayoutEffect)(() => {
    const el = scrollerRef.current;
    if (!el) return;
    let scrollbarAnimation;
    const updateScrollbar = () => {
      cancelAnimationFrame(scrollbarAnimation);
      scrollbarAnimation = requestAnimationFrame(() => {
        var _a, _b;
        const { clientHeight, scrollHeight } = el;
        setScrollRatio(clientHeight / scrollHeight);
        memoizedProps.current = {
          clientHeight,
          scrollHeight,
          trackHeight: (_b = (_a = trackRef.current) == null ? void 0 : _a.clientHeight) != null ? _b : 0
        };
      });
    };
    const resizeObserver = new ResizeObserver(updateScrollbar);
    const observeScroller = () => {
      resizeObserver.observe(el);
      for (const node of el.children) {
        resizeObserver.observe(node);
      }
    };
    observeScroller();
    const mutationObserver = new MutationObserver(() => {
      resizeObserver.disconnect();
      observeScroller();
      updateScrollbar();
    });
    mutationObserver.observe(el, { childList: true, subtree: true });
    window.addEventListener("resize", updateScrollbar);
    updateScrollbar();
    return () => {
      cancelAnimationFrame(scrollbarAnimation);
      window.removeEventListener("resize", updateScrollbar);
      resizeObserver.disconnect();
      mutationObserver.disconnect();
    };
  }, [scrollerRef]);
  (0, import_react.useLayoutEffect)(() => {
    if (!disabled) return;
    const el = scrollerRef.current;
    if (!el) return;
    const onWheel = (e) => e.preventDefault();
    el.addEventListener("wheel", onWheel, { passive: false });
    return () => {
      el.removeEventListener("wheel", onWheel);
    };
  }, [scrollerRef, disabled]);
  const onScroll = (0, import_react.useCallback)(() => {
    if (scrollRatio === 1) return;
    const el = scrollerRef.current;
    const track = trackRef.current;
    if (!el || !track) return;
    cancelAnimationFrame(trackAnimationRef.current);
    trackAnimationRef.current = requestAnimationFrame(() => {
      if (!memoizedProps.current) return;
      const { clientHeight, scrollHeight, trackHeight } = memoizedProps.current;
      const ratio = el.scrollTop / (scrollHeight - clientHeight);
      const y = ratio * (clientHeight - trackHeight);
      track.style.transform = `translateY(${y}px)`;
    });
  }, [scrollerRef, scrollRatio]);
  const moveTrack = (0, import_react.useCallback)(
    (e) => {
      const el = scrollerRef.current;
      if (!el) return;
      let moveAnimation;
      let lastPageY = e.pageY;
      let lastScrollTop = el.scrollTop;
      setIsDraggingTrack(true);
      const drag = ({ pageY }) => {
        cancelAnimationFrame(moveAnimation);
        moveAnimation = requestAnimationFrame(() => {
          const delta = pageY - lastPageY;
          lastScrollTop += delta / scrollRatio;
          lastPageY = pageY;
          el.scrollTop = lastScrollTop;
        });
      };
      const stop = () => {
        setIsDraggingTrack(false);
        window.removeEventListener("mousemove", drag);
      };
      window.addEventListener("mousemove", drag);
      window.addEventListener("mouseup", stop, { once: true });
    },
    [scrollerRef, scrollRatio]
  );
  const wrapperProps = {
    style: {
      marginLeft: `-${SCROLLBAR_WIDTH}px`
    }
  };
  const scrollerProps = {
    ref: scrollerRef,
    onScroll: disabled ? void 0 : onScroll,
    style: {
      right: `-${SCROLLBAR_WIDTH}px`,
      padding: `0 ${SCROLLBAR_WIDTH}px 0 0`,
      width: `calc(100% + ${OS_SCROLLBAR_WIDTH}px)`
    }
  };
  const trackProps = {
    ref: trackRef,
    onMouseDown: disabled ? void 0 : moveTrack,
    style: {
      right: isDraggingTrack ? 1 : void 0,
      width: isDraggingTrack ? 10 : void 0,
      height: `${scrollRatio * 100}%`,
      opacity: isDraggingTrack ? 1 : void 0,
      display: disabled || scrollRatio === 1 ? "none" : void 0
    }
  };
  return [wrapperProps, scrollerProps, trackProps];
}

// src/index.module.css
var src_default = {
  _main_ubzcz_1: "src__main_ubzcz_1",
  _track_ubzcz_5: "src__track_ubzcz_5",
  _wrapper_ubzcz_9: "src__wrapper_ubzcz_9",
  _inner_ubzcz_15: "src__inner_ubzcz_15"
};

// src/index.tsx
var cx = (...args) => args.filter(Boolean).join(" ");
var CustomScroller = (0, import_react2.forwardRef)(
  ({ scrollDisabled = false, className, innerClassName, children, ...props }, ref) => {
    const [wrapperProps, scrollerProps, trackProps] = useCustomScroller(
      ref !== null && "current" in ref ? ref : void 0,
      { disabled: scrollDisabled }
    );
    return /* @__PURE__ */ React.createElement("div", { className: cx(className, src_default.main), ...props }, /* @__PURE__ */ React.createElement("div", { className: src_default.wrapper, ...wrapperProps }, /* @__PURE__ */ React.createElement("div", { className: cx(innerClassName, src_default.inner), ...scrollerProps }, children)), /* @__PURE__ */ React.createElement("div", { className: src_default.track, ...trackProps }));
  }
);
CustomScroller.displayName = "CustomScroller";
var src_default2 = CustomScroller;
//# sourceMappingURL=index.js.map